@@ -9,8 +9,9 @@ import google.protobuf.json_format as parser
 import requests
 import threading
 import typing
-import random, queue
+import random
 from concurrent.futures import Future, ThreadPoolExecutor, as_completed
+from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PublicKey
 
 from .. import diem_types, utils
 from . import jsonrpc_pb2 as rpc
@@ -55,7 +56,7 @@ class WaitForTransactionTimeout(Exception):
     pass
 
 
-class AccountNotFoundError(Exception):
+class AccountNotFoundError(ValueError):
     pass
 
 
@@ -156,7 +157,7 @@ class RequestWithBackups(RequestStrategy):
     def _fallback_to_backup(self, primary: Future, backup: Future) -> typing.Dict[str, typing.Any]:
         try:
             return primary.result()
-        except Exception as e:
+        except Exception:
             return backup.result()
 
     def _first_success(self, primary: Future, backup: Future) -> typing.Dict[str, typing.Any]:
@@ -171,7 +172,7 @@ class RequestWithBackups(RequestStrategy):
 class Client:
     """Diem JSON-RPC API client
 
-    [SPEC](https://github.com/libra/libra/blob/master/json-rpc/json-rpc-spec.md)
+    [SPEC](https://github.com/diem/diem/blob/master/json-rpc/json-rpc-spec.md)
     """
 
     def __init__(
@@ -205,10 +206,7 @@ class Client:
         could not find the account by the parent_vasp_address found in ChildVASP account.
         """
 
-        account = self.get_account(vasp_account_address)
-        if account is None:
-            hex = utils.account_address_hex(vasp_account_address)
-            raise AccountNotFoundError(f"account not found by address: {hex}")
+        account = self.must_get_account(vasp_account_address)
 
         if account.role.type == constants.ACCOUNT_ROLE_PARENT_VASP:
             return account
@@ -218,6 +216,33 @@ class Client:
         hex = utils.account_address_hex(vasp_account_address)
         raise ValueError(f"given account address({hex}) is not a VASP account: {account}")
 
+    def get_base_url_and_compliance_key(
+        self, account_address: typing.Union[diem_types.AccountAddress, str]
+    ) -> typing.Tuple[str, Ed25519PublicKey]:
+        """get base_url and compliance key
+
+        ParentVASP or Designated Dealer account role has base_url and compliance key setup, which
+        are used for offchain API communication.
+        """
+        account = self.must_get_account(account_address)
+
+        if account.role.compliance_key and account.role.base_url:
+            key = Ed25519PublicKey.from_public_bytes(bytes.fromhex(account.role.compliance_key))
+            return (account.role.base_url, key)
+        if account.role.parent_vasp_address:
+            return self.get_base_url_and_compliance_key(account.role.parent_vasp_address)
+
+        raise ValueError(f"could not find base_url and compliance_key from account: {account}")
+
+    def must_get_account(self, account_address: typing.Union[diem_types.AccountAddress, str]) -> rpc.Account:
+        """must_get_account raises AccountNotFoundError if account could not be found by given address"""
+
+        account = self.get_account(account_address)
+        if account is None:
+            hex = utils.account_address_hex(account_address)
+            raise AccountNotFoundError(f"account not found by address: {hex}")
+        return account
+
     def get_account_sequence(self, account_address: typing.Union[diem_types.AccountAddress, str]) -> int:
         """get on-chain account sequence number
 
@@ -276,7 +301,7 @@ class Client:
     ) -> typing.Optional[rpc.Metadata]:
         """get block metadata
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_metadata.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_metadata.md)
         """
 
         params = [int(version)] if version else []
@@ -285,7 +310,7 @@ class Client:
     def get_currencies(self) -> typing.List[rpc.CurrencyInfo]:
         """get currencies
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_currencies.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_currencies.md)
         """
 
         return self.execute("get_currencies", [], _parse_list(lambda: rpc.CurrencyInfo()))
@@ -296,7 +321,7 @@ class Client:
         """get on-chain account information
 
         Returns None if account not found
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_account.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_account.md)
         """
 
         address = utils.account_address_hex(account_address)
@@ -312,7 +337,7 @@ class Client:
 
         Returns None if transaction is not found
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_account_transaction.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_account_transaction.md)
         """
 
         address = utils.account_address_hex(account_address)
@@ -330,7 +355,7 @@ class Client:
 
         Returns empty list if no transactions found
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_account_transactions.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_account_transactions.md)
         """
 
         address = utils.account_address_hex(account_address)
@@ -347,7 +372,7 @@ class Client:
 
         Returns empty list if no transactions found
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_transactions.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_transactions.md)
         """
 
         params = [int(start_version), int(limit), bool(include_events)]
@@ -358,7 +383,7 @@ class Client:
 
         Returns empty list if no events found
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_get_events.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_get_events.md)
         """
 
         params = [event_stream_key, int(start), int(limit)]
@@ -391,7 +416,7 @@ class Client:
 
         Pass in `raise_stale_response=True` to raise StaleResponseError and handle it by yourself.
 
-        See [JSON-RPC API Doc](https://github.com/libra/libra/blob/master/json-rpc/docs/method_submit.md)
+        See [JSON-RPC API Doc](https://github.com/diem/diem/blob/master/json-rpc/docs/method_submit.md)
         """
 
         if isinstance(txn, diem_types.SignedTransaction):
@@ -552,9 +577,9 @@ class Client:
         # check stable response before check jsonrpc error
         try:
             self.update_last_known_state(
-                json.get("libra_chain_id"),
-                json.get("libra_ledger_version"),
-                json.get("libra_ledger_timestampusec"),
+                json.get("diem_chain_id"),
+                json.get("diem_ledger_version"),
+                json.get("diem_ledger_timestampusec"),
             )
         except StaleResponseError as e:
             if not ignore_stale_response:
